unit TrafficLight.MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, TrafficLight.Model,
  Vcl.StdCtrls, expr;

type
  TForm1 = class(TForm)
    shapeYellow: TShape;
    Memo1: TMemo;
    btnStart: TButton;
    btnStop: TButton;
    shapeRed: TShape;
    shapeGreen: TShape;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    FParser: TExprParser;
    FInitEvents, FLoopEvents: TEventList;

    FRedLight, FYellowLight, FGreenLight: TCustomLight;

    FBreak: Boolean;
    procedure DoWork;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.btnStartClick(Sender: TObject);
var
  StrStream: TStringStream;
begin
  FInitEvents.Clear;
  FLoopEvents.Clear;

  StrStream := TStringStream.Create;
  try
    StrStream.WriteString(AnsiUpperCase(Memo1.Text));
    StrStream.Position := 0;
    try
      FParser.parse(StrStream, FRedLight, FYellowLight, FGreenLight, FInitEvents, FLoopEvents);
    except
      on E: EExprParserException do
        ShowMessage(E.Message);
    end;
  finally
    StrStream.Free;
  end;

  DoWork;
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
  FBreak := True;
end;

procedure TForm1.DoWork;
var
  I: Integer;
begin
  FBreak := False;
  btnStart.Enabled := False;
  btnStop.Enabled := True;

  for I := 0 to FInitEvents.Count-1 do
  begin
    FInitEvents[I].Execute;
    Application.ProcessMessages;
  end;

  while not FBreak do
    for I := 0 to FLoopEvents.Count-1 do
    begin
      FLoopEvents[I].Execute;
      Application.ProcessMessages;
    end;

  btnStart.Enabled := True;
  btnStop.Enabled := False;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FBreak := True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FRedLight := TCustomLight.Create(shapeRed, clRed, clBlack);
  FYellowLight := TCustomLight.Create(shapeYellow, clYellow, clBlack);
  FGreenLight := TCustomLight.Create(shapeGreen, clLime, clBlack);

  FInitEvents := TEventList.Create(True);
  FLoopEvents := TEventList.Create(True);

  FParser := TExprParser.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FParser.Free;

  FLoopEvents.Free;
  FInitEvents.Free;

  FRedLight.Free;
  FYellowLight.Free;
  FGreenLight.Free;
end;

end.
