unit TrafficLight.Model;

interface

uses
  Winapi.Windows, Generics.Collections, Vcl.Graphics, Vcl.ExtCtrls;

type
  TCustomLight = class
  private
    FColorOn, FColorOff: TColor;
    FShape: TShape;
  public
    constructor Create(AShape: TShape; AColorOn, AColorOff: TColor); reintroduce;

    procedure TurnOn;
    procedure TurnOff;
  end;

  TCustomEvent = class
  public
    procedure Execute; virtual; abstract;
  end;

  TEventList = TObjectList<TCustomEvent>;

  TLightOnEvent = class(TCustomEvent)
  private
    FLight: TCustomLight;
  public
    constructor Create(ALight: TCustomLight); reintroduce;
    procedure Execute; override;
  end;

  TLightOffEvent = class(TCustomEvent)
  private
    FLight: TCustomLight;
  public
    constructor Create(ALight: TCustomLight); reintroduce;
    procedure Execute; override;
  end;

  TWaitEvent = class(TCustomEvent)
  private
    FWaitTime: Cardinal;
  public
    constructor Create(AWaitTime: Cardinal); reintroduce;
    procedure Execute; override;
  end;

implementation

{ TCustomLight }

constructor TCustomLight.Create(AShape: TShape; AColorOn, AColorOff: TColor);
begin
  inherited Create;
  FShape := AShape;
  FColorOn := AColorOn;
  FColorOff := AColorOff;
end;

procedure TCustomLight.TurnOff;
begin
  FShape.Brush.Color := FColorOff;
end;

procedure TCustomLight.TurnOn;
begin
  FShape.Brush.Color := FColorOn;
end;

{ TLightOnEvent }

constructor TLightOnEvent.Create(ALight: TCustomLight);
begin
  inherited Create;
  FLight := ALight;
end;

procedure TLightOnEvent.Execute;
begin
  FLight.TurnOn;
end;

{ TLightOffEvent }

constructor TLightOffEvent.Create(ALight: TCustomLight);
begin
  inherited Create;
  FLight := ALight;
end;

procedure TLightOffEvent.Execute;
begin
  FLight.TurnOff;
end;

{ TWaitEvent }

constructor TWaitEvent.Create(AWaitTime: Cardinal);
begin
  inherited Create;
  FWaitTime := AWaitTime;
end;

procedure TWaitEvent.Execute;
begin
  Sleep(FWaitTime*1000);
end;

end.
