
unit expr;

interface

uses
	SysUtils,
	Classes,
	yacclib,
	lexlib,
        uStreamLexer,
        TrafficLight.Model;



const NUM = 257;
const RED = 258;
const YELLOW = 259;
const GREEN = 260;
const INIT = 261;
const ENDINIT = 262;
const LOOP = 263;
const ENDLOOP = 264;
const ON = 265;
const OFF = 266;
const WAIT = 267;
const ILLEGAL = 268;

// If you have defined your own YYSType then put an empty  %union { } in
// your .y file. Or you can put your type definition within the curly braces.
type YYSType = record
                 yyCardinal : Cardinal;
                 yyTCustomEvent : TCustomEvent;
                 yyTCustomLight : TCustomLight;
                 yyTEventList : TEventList;
               end(*YYSType*);
// source: yyparse.cod line# 2

var yylval : YYSType;

type
  TExprLexer = class(TStreamLexer)
  public
    function parse() : integer; override;
  end;

  TWriteCallback = procedure(Value: Real) of object;
  EExprParserException = class(Exception);
  TExprParser = class
  private
    FTemporaryList: TEventList;

    FRedLight, FYellowLight, FGreenLight: TCustomLight;
    FInitList, FLoopList: TEventList;
  public
    constructor Create;
    destructor Destroy; override;
    function parse(AStream: TStream; ARedLight, AYellowLight, AGreenLight: TCustomLight; AInitList, ALoopList: TEventList) : integer; reintroduce;
  end;

implementation

constructor TExprParser.Create;
begin
  inherited;
  FTemporaryList := TEventList.Create(False);
end;

destructor TExprParser.Destroy;
begin
  FTemporaryList.Free;
  inherited;
end;

function TExprParser.parse(AStream: TStream; ARedLight, AYellowLight, AGreenLight: TCustomLight; AInitList, ALoopList: TEventList) : integer;

var 
  lexer : TExprLexer;
  yystate, yysp, yyn : Integer;
  yys : array [1..yymaxdepth] of Integer;
  yyv : array [1..yymaxdepth] of YYSType;
  yyval : YYSType;

procedure yyaction ( yyruleno : Integer );
  (* local definitions: *)
// source: yyparse.cod line# 51
begin
  (* actions: *)
  case yyruleno of
1 : begin
       end;
2 : begin
         // source: expr.y line#33
         yyaccept; 
       end;
3 : begin
         // source: expr.y line#37
         FInitList.AddRange(FTemporaryList.ToArray); FTemporaryList.Clear; 
       end;
4 : begin
         // source: expr.y line#40
         FLoopList.AddRange(FTemporaryList.ToArray); FTemporaryList.Clear 
       end;
5 : begin
       end;
6 : begin
         // source: expr.y line#44
         FTemporaryList.Add(yyv[yysp-0].yyTCustomEvent); 
       end;
7 : begin
         // source: expr.y line#47
         yyval.yyTCustomEvent := TWaitEvent.Create(yyv[yysp-0].yyCardinal); 
       end;
8 : begin
         // source: expr.y line#48
         yyval.yyTCustomEvent := TLightOnEvent.Create(yyv[yysp-1].yyTCustomLight); 
       end;
9 : begin
         // source: expr.y line#49
         yyval.yyTCustomEvent := TLightOffEvent.Create(yyv[yysp-1].yyTCustomLight); 
       end;
10 : begin
         // source: expr.y line#52
         yyval.yyTCustomLight := FRedLight; 
       end;
11 : begin
         // source: expr.y line#53
         yyval.yyTCustomLight := FYellowLight; 
       end;
12 : begin
         // source: expr.y line#54
         yyval.yyTCustomLight := FGreenLight; 
       end;
// source: yyparse.cod line# 55
  end;
end(*yyaction*);

(* parse table: *)

type YYARec = record
                sym, act : Integer;
              end;
     YYRRec = record
                len, sym : Integer;
              end;

const

yynacts   = 17;
yyngotos  = 9;
yynstates = 19;
yynrules  = 12;
yymaxtoken = 268;

yya : array [1..yynacts] of YYARec = (
{ 0: }
  ( sym: 261; act: 3 ),
  ( sym: 0; act: -1 ),
{ 1: }
  ( sym: 263; act: 5 ),
{ 2: }
  ( sym: 0; act: 0 ),
{ 3: }
{ 4: }
{ 5: }
{ 6: }
  ( sym: 258; act: 10 ),
  ( sym: 259; act: 11 ),
  ( sym: 260; act: 12 ),
  ( sym: 262; act: 13 ),
  ( sym: 267; act: 14 ),
{ 7: }
  ( sym: 258; act: 10 ),
  ( sym: 259; act: 11 ),
  ( sym: 260; act: 12 ),
  ( sym: 264; act: 15 ),
  ( sym: 267; act: 14 ),
{ 8: }
  ( sym: 265; act: 16 ),
  ( sym: 266; act: 17 ),
{ 9: }
{ 10: }
{ 11: }
{ 12: }
{ 13: }
{ 14: }
  ( sym: 257; act: 18 )
{ 15: }
{ 16: }
{ 17: }
{ 18: }
);

yyg : array [1..yyngotos] of YYARec = (
{ 0: }
  ( sym: -5; act: 1 ),
  ( sym: -4; act: 2 ),
{ 1: }
  ( sym: -6; act: 4 ),
{ 2: }
{ 3: }
  ( sym: -7; act: 6 ),
{ 4: }
{ 5: }
  ( sym: -7; act: 7 ),
{ 6: }
  ( sym: -3; act: 8 ),
  ( sym: -2; act: 9 ),
{ 7: }
  ( sym: -3; act: 8 ),
  ( sym: -2; act: 9 )
{ 8: }
{ 9: }
{ 10: }
{ 11: }
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
);

yyd : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 0,
{ 3: } -5,
{ 4: } -2,
{ 5: } -5,
{ 6: } 0,
{ 7: } 0,
{ 8: } 0,
{ 9: } -6,
{ 10: } -10,
{ 11: } -11,
{ 12: } -12,
{ 13: } -3,
{ 14: } 0,
{ 15: } -4,
{ 16: } -8,
{ 17: } -9,
{ 18: } -7
);

yyal : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 3,
{ 2: } 4,
{ 3: } 5,
{ 4: } 5,
{ 5: } 5,
{ 6: } 5,
{ 7: } 10,
{ 8: } 15,
{ 9: } 17,
{ 10: } 17,
{ 11: } 17,
{ 12: } 17,
{ 13: } 17,
{ 14: } 17,
{ 15: } 18,
{ 16: } 18,
{ 17: } 18,
{ 18: } 18
);

yyah : array [0..yynstates-1] of Integer = (
{ 0: } 2,
{ 1: } 3,
{ 2: } 4,
{ 3: } 4,
{ 4: } 4,
{ 5: } 4,
{ 6: } 9,
{ 7: } 14,
{ 8: } 16,
{ 9: } 16,
{ 10: } 16,
{ 11: } 16,
{ 12: } 16,
{ 13: } 16,
{ 14: } 17,
{ 15: } 17,
{ 16: } 17,
{ 17: } 17,
{ 18: } 17
);

yygl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 3,
{ 2: } 4,
{ 3: } 4,
{ 4: } 5,
{ 5: } 5,
{ 6: } 6,
{ 7: } 8,
{ 8: } 10,
{ 9: } 10,
{ 10: } 10,
{ 11: } 10,
{ 12: } 10,
{ 13: } 10,
{ 14: } 10,
{ 15: } 10,
{ 16: } 10,
{ 17: } 10,
{ 18: } 10
);

yygh : array [0..yynstates-1] of Integer = (
{ 0: } 2,
{ 1: } 3,
{ 2: } 3,
{ 3: } 4,
{ 4: } 4,
{ 5: } 5,
{ 6: } 7,
{ 7: } 9,
{ 8: } 9,
{ 9: } 9,
{ 10: } 9,
{ 11: } 9,
{ 12: } 9,
{ 13: } 9,
{ 14: } 9,
{ 15: } 9,
{ 16: } 9,
{ 17: } 9,
{ 18: } 9
);

yyr : array [1..yynrules] of YYRRec = (
{ 1: } ( len: 0; sym: -4 ),
{ 2: } ( len: 2; sym: -4 ),
{ 3: } ( len: 3; sym: -5 ),
{ 4: } ( len: 3; sym: -6 ),
{ 5: } ( len: 0; sym: -7 ),
{ 6: } ( len: 2; sym: -7 ),
{ 7: } ( len: 2; sym: -2 ),
{ 8: } ( len: 2; sym: -2 ),
{ 9: } ( len: 2; sym: -2 ),
{ 10: } ( len: 1; sym: -3 ),
{ 11: } ( len: 1; sym: -3 ),
{ 12: } ( len: 1; sym: -3 )
);

// source: yyparse.cod line# 60

const _error = 256; (* error token *)

function yyact(state, sym : Integer; var act : Integer) : Boolean;
  (* search action table *)
  var k : Integer;
  begin
    k := yyal[state];
    while (k<=yyah[state]) and (yya[k].sym<>sym) do inc(k);
    if k>yyah[state] then
      yyact := false
    else
      begin
        act := yya[k].act;
        yyact := true;
      end;
  end(*yyact*);

function yygoto(state, sym : Integer; var nstate : Integer) : Boolean;
  (* search goto table *)
  var k : Integer;
  begin
    k := yygl[state];
    while (k<=yygh[state]) and (yyg[k].sym<>sym) do inc(k);
    if k>yygh[state] then
      yygoto := false
    else
      begin
        nstate := yyg[k].act;
        yygoto := true;
      end;
  end(*yygoto*);

function yycharsym(i : Integer) : String;
begin
  if (i >= 1) and (i <= 255) then
    begin
      if i < 32 then
        begin
          if i = 9 then
            Result := #39'\t'#39
          else if i = 10 then
            Result := #39'\f'#39
          else if i = 13 then
            Result := #39'\n'#39
          else
            Result := #39'\0x' + IntToHex(i,2) + #39;
        end
      else
        Result := #39 + Char(i) + #39;
      Result := ' literal ' + Result;
    end
  else
    begin
      if i < -1 then
        Result := ' unknown'
      else if i = -1 then
        Result := ' token $accept'
      else if i = 0 then
        Result := ' token $eof'
      else if i = 256 then
        Result := ' token $error'
{$ifdef yyextradebug}
      else if i <= yymaxtoken then
        Result := ' token ' + yytokens[yychar].tokenname
      else
        Result := ' unknown token';
{$else}
      else
        Result := ' token';
{$endif}
    end;
  Result := Result + ' ' + IntToStr(yychar);
end;

label parse, next, error, errlab, shift, reduce, accept, abort;

begin(*yyparse*)

  (* initialize: *)
  lexer := TExprLexer.Create(AStream, nil);

  FRedLight := ARedLight;
  FYellowLight := AYellowLight;
  FGreenLight := AGreenLight;
  FInitList := AInitList;
  FLoopList := ALoopList;

  try

  yystate := 0; yychar := -1; yynerrs := 0; yyerrflag := 0; yysp := 0;

parse:

  (* push state and value: *)

  inc(yysp);
  if yysp>yymaxdepth then
    begin
      raise EExprParserException.Create('yyparse stack overflow');
      goto abort;
    end;
  yys[yysp] := yystate; yyv[yysp] := yyval;

next:

  if (yyd[yystate]=0) and (yychar=-1) then
    (* get next symbol *)
    begin
      yychar := lexer.parse(); if yychar<0 then yychar := 0;
    end;

  {$IFDEF YYDEBUG}writeln('state ', yystate, yycharsym(yychar));{$ENDIF}

  (* determine parse action: *)

  yyn := yyd[yystate];
  if yyn<>0 then goto reduce; (* simple state *)

  (* no default action; search parse table *)

  if not yyact(yystate, yychar, yyn) then goto error
  else if yyn>0 then                      goto shift
  else if yyn<0 then                      goto reduce
  else                                    goto accept;

error:

  (* error; start error recovery: *)

  if yyerrflag=0 then raise EExprParserException.Create('syntax error');

errlab:

  if yyerrflag=0 then inc(yynerrs);     (* new error *)

  if yyerrflag<=2 then                  (* incomplete recovery; try again *)
    begin
      yyerrflag := 3;
      (* uncover a state with shift action on error token *)
      while (yysp>0) and not ( yyact(yys[yysp], _error, yyn) and
                               (yyn>0) ) do
        begin
          {$IFDEF YYDEBUG}
            if yysp>1 then
              writeln('error recovery pops state ', yys[yysp], ', uncovers ',
                      yys[yysp-1])
            else
              writeln('error recovery fails ... abort');
          {$ENDIF}
          dec(yysp);
        end;
      if yysp=0 then goto abort; (* parser has fallen from stack; abort *)
      yystate := yyn;            (* simulate shift on error *)
      goto parse;
    end
  else                                  (* no shift yet; discard symbol *)
    begin
      {$IFDEF YYDEBUG}writeln('error recovery discards ' + yycharsym(yychar));{$ENDIF}
      if yychar=0 then goto abort; (* end of input; abort *)
      yychar := -1; goto next;     (* clear lookahead char and try again *)
    end;

shift:

  (* go to new state, clear lookahead character: *)

  yystate := yyn; yychar := -1; yyval := yylval;
  if yyerrflag>0 then dec(yyerrflag);

  goto parse;

reduce:

  (* execute action, pop rule from stack, and go to next state: *)

  {$IFDEF YYDEBUG}writeln('reduce ' + IntToStr(-yyn) {$IFDEF YYEXTRADEBUG} + ' rule ' + yyr[-yyn].symname {$ENDIF});{$ENDIF}

  yyflag := yyfnone; yyaction(-yyn);
  dec(yysp, yyr[-yyn].len);
  if yygoto(yys[yysp], yyr[-yyn].sym, yyn) then yystate := yyn;

  (* handle action calls to yyaccept, yyabort and yyerror: *)

  case yyflag of
    yyfaccept : goto accept;
    yyfabort  : goto abort;
    yyferror  : goto errlab;
  end;

  goto parse;

accept:

  Result := 0; exit;

abort:

  Result := 1; exit;
finally
  lexer.Free;
end;
end(*yyparse*);


{$I exprlex.pas}

end.